-----------------------------------------------------------------------------------------------
-- Client Lua Script for Rivalry
-- Copyright (c) NCsoft. All rights reserved
-- Made by Aapjuh(Pvp-Zhur)
-- Special Thanks to "Never" and "Jezex"
-----------------------------------------------------------------------------------------------
 
require "Apollo"
require "Window"
require "Unit"
require "GameLib"
require "ChatSystemLib"
require "ChatChannelLib"
require "GroupLib"
require "FriendshipLib"
require "HousingLib"
 
-----------------------------------------------------------------------------------------------
-- Rivalry Module Definition
-----------------------------------------------------------------------------------------------
local Rivalry = {} 

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------

function Rivalry:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function Rivalry:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
-----------------------------------------------------------------------------------------------
-- Rivalry OnLoad
-----------------------------------------------------------------------------------------------

function Rivalry:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Rivalry.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	Apollo.LoadSprites("Sprite.xml", "Sprite")
	Apollo.RegisterEventHandler("CombatLogDamage","OnCombatLogDamage", self)
	Apollo.RegisterSlashCommand("rivalryno", "OnRivalryOn", self)
	Apollo.RegisterSlashCommand("rivalryok", "OnOK", self)
	myMurderTimer = {}
    myMurderTimer.Timer = ApolloTimer.Create(3, false, "OnMurder", self)
	myMurderTimer.CarryArg = nil
	myRevengeTimer = {}
	myRevengeTimer.Timer = ApolloTimer.Create(3, false, "OnRevenge", self)
	myRevengeTimer.CarryArg = nil
	myChatLocaleText = {}
	Rivalry:GetLocale()
end

function Rivalry:GetLocale()
	local strCancel = Apollo.GetString(1)
	-- German
	if strCancel == "Abbrechen" then
		myZoneBlackList = {
		"PvP-Arena", "Hallen der Blutgeschworenen", "Walatiki-Tempel",
		}
		myChatLocaleText.AddRival = "/rivale "
		myChatLocaleText.UnRival = "/keinrivale "
		myChatLocaleText.RivalNote = "/freundNotiz "
		myChatLocaleText.RivalAddLine1 = "Hinzugefügt "
		myChatLocaleText.RivalAddLine2 = " als Rivalen"
		myChatLocaleText.RevengeLine1 = "Sie rächte sich an "
		return
	end
	-- French
	if strCancel == "Annuler" then
		myZoneBlackList = {
		"Arène JcJ", "Château de Juresang", "Temple de Walatiki",
		}
		myChatLocaleText.AddRival = "/rival "
		myChatLocaleText.UnRival = "/pasrival "
		myChatLocaleText.RivalNote = "/aminote "
		myChatLocaleText.RivalAddLine1 = "Ajouté "
		myChatLocaleText.RivalAddLine2 = " comme Rival"
		myChatLocaleText.RevengeLine1 = "Vous avez pris sa revanche sur "
		return
	end
	-- English and Other
	myZoneBlackList = {
		"PvP Arena", "Halls of the Bloodsworn", "Walatiki Temple",
		}
	myChatLocaleText.AddRival = "/rival "
	myChatLocaleText.UnRival = "/unrival "
	myChatLocaleText.RivalNote = "/friendnote "
	myChatLocaleText.RivalAddLine1 = "Added "
	myChatLocaleText.RivalAddLine2 = " as Rival"
	myChatLocaleText.RevengeLine1 = "You took revenge on "
	return
end

-----------------------------------------------------------------------------------------------
-- Rivalry OnDocLoaded
-----------------------------------------------------------------------------------------------
function Rivalry:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.myForm = Apollo.LoadForm(self.xmlDoc, "RivalryForm", nil, self)
		if self.myForm == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.myForm:Show(false)
		local tDisplaySize = Apollo.GetDisplaySize()
		local nCenterX, nCenterY = tDisplaySize.nWidth / 2, tDisplaySize.nHeight / 4
		local nMainX = 295
		local nMainY = 60
		self.myForm:SetAnchorOffsets(nCenterX - nMainX, nCenterY - nMainY, nCenterX + nMainX, nCenterY + nMainY)
		myShowOrCloseWindow = self
	end
end

-----------------------------------------------------------------------------------------------
-- Rivalry Functions
-----------------------------------------------------------------------------------------------

-- on SlashCommand "/Rivalryno"
function Rivalry:OnRivalryOn()
	Print("Used for Debugging ignore the /command")
end

-- on SlashCommand "/Rivalryok"
function Rivalry:OnOK()
	Print("Used for Debugging ignore the /command")
end

-----------------------------------------------------------------------------------------------
-- RivalryForm Functions
-----------------------------------------------------------------------------------------------

function Rivalry:isZoneBlackList(ZoneName)
	if HousingLib:IsHousingWorld() then
		return true
	end
	if HousingLib:IsWarplotResidence() then
		return true
	end
	for i = 1, table.getn(myZoneBlackList) do
		if myZoneBlackList[i] == ZoneName then
			return true
		end
	end
	return false
end

function Rivalry:OnCombatLogDamage(CombatInfo)
	--Check Blacklist
	local myCurrentZoneName = GameLib.GetCurrentZoneMap()
	if myCurrentZoneName == nil then
		return
	end
	if Rivalry:isZoneBlackList(myCurrentZoneName.strName) then
		return
	end
	
	if CombatInfo.nOverkill > 0 then
		local MyMurdererClean = nil
		local MyRevengeClean = nil
		
		if CombatInfo.unitCaster ~= nil and CombatInfo.unitTarget ~= nil then
			if CombatInfo.unitCaster:IsACharacter() and CombatInfo.unitTarget:IsACharacter() then
				local faction1 = CombatInfo.unitTarget:GetFaction()
				local faction2 = CombatInfo.unitCaster:GetFaction()
				if faction1 ~= faction2 then
					if GameLib.GetPlayerUnit():GetName() == CombatInfo.unitCaster:GetName() then
						MyRevengeClean = CombatInfo.unitTarget:GetName()
						Rivalry:MyRevenge(MyRevengeClean)
					else
						MyMurdererClean = CombatInfo.unitCaster:GetName()
						Rivalry:MyMurderer(MyMurdererClean)
					end
				end
			end
		end
	end
end

function Rivalry:MyMurderer(MyMurdererClean)
	if MyMurdererClean == nil then
		return
	end

	local RivalList = FriendshipLib.GetList()
	for i = 1, table.getn(RivalList) do
		if RivalList[i].strCharacterName == MyMurdererClean then
			local NoteID = RivalList[i].nId
			if tonumber(RivalList[i].strNote) ~= nil then
				local NewNote = RivalList[i].strNote + 1
				FriendshipLib.SetNote(NoteID, NewNote)
				return
			end
		end
	end
	Print(myChatLocaleText.RivalAddLine1 ..MyMurdererClean ..myChatLocaleText.RivalAddLine2 .."..!")
	ChatSystemLib.Command(myChatLocaleText.AddRival ..MyMurdererClean)
	myMurderTimer.CarryArg = MyMurdererClean
	myMurderTimer.Timer:Start()
end

function Rivalry:MyRevenge(MyRevengeClean)
	if MyRevengeClean == nil then
		return
	end

	local RivalList = FriendshipLib.GetList()
	for i = 1, table.getn(RivalList) do
		if RivalList[i].strCharacterName == MyRevengeClean then
			local NoteID = RivalList[i].nId
			if tonumber(RivalList[i].strNote) ~= nil then
				local NewNote = RivalList[i].strNote - 1
				if NewNote == 0 then
					Print(myChatLocaleText.RevengeLine1 ..MyRevengeClean)
					myShowOrCloseWindow.myForm:Show(true)
					myRevengeTimer.CarryArg = MyRevengeClean
					myRevengeTimer.Timer:Start()
					return
				end
				FriendshipLib.SetNote(NoteID, NewNote)
				return
			end
		end
	end
end

function Rivalry:OnMurder()
	if myMurderTimer.CarryArg ~= nil then
		ChatSystemLib.Command(myChatLocaleText.RivalNote ..myMurderTimer.CarryArg .." 1")
		myMurderTimer.Timer:Stop()
	end
end

function Rivalry:OnRevenge()
	if myRevengeTimer.CarryArg ~= nil then
		myShowOrCloseWindow.myForm:Show(false)
		ChatSystemLib.Command(myChatLocaleText.UnRival ..myRevengeTimer.CarryArg)
		myRevengeTimer.Timer:Stop()
	end
end

-----------------------------------------------------------------------------------------------
-- Rivalry Instance
-----------------------------------------------------------------------------------------------

local RivalryInst = Rivalry:new()
RivalryInst:Init()
